# Get SMART values for windows
#
# - Install smartmontools
# -- Download smartmontools-x.x-x.win32-setup.exe from http://sourceforge.net/projects/smartmontools/files/smartmontools/
# -- Run: smartmontools-x.x-x.win32-setup.exe
# -- Default install path: C:\Program Files (x86)\smartmontools\bin\
#
# - Open cmd.exe
# -- Run: powershell Set-ExecutionPolicy RemoteSigned
#
# - Change C:\zabbix\zabbix_agentd.conf
# -- Timeout=10
# -- Run: zabbix_agentd.exe --config C:\zabbix\zabbix_agentd.conf --stop
# -- Run: zabbix_agentd.exe --config C:\zabbix\zabbix_agentd.conf --start
#
# Author: DEX

$script:SMARTCTL_64="C:\Program Files\smartmontools\bin\smartctl.exe"
$script:SMARTCTL_32="C:\Program Files (x86)\smartmontools\bin\smartctl.exe"

function GetSMARTCTL(){
	if( Test-Path -path $SMARTCTL_64 ) {
		return $SMARTCTL_64
	}elseif( Test-Path -path $SMARTCTL_32 ){
		return $SMARTCTL_32
	}else{
		return ""
	}
}

function ExecCmd([string]$strCmd) {
	$p = [System.Diagnostics.Process];
	$pi = [System.Diagnostics.ProcessStartInfo];
	$output = [string];

	$pi = New-Object System.Diagnostics.ProcessStartInfo;

	$pi.FileName = [System.Environment]::GetEnvironmentVariable("ComSpec");
	$pi.Arguments = "/c " + $strCmd;
	$pi.UseShellExecute = $false;
	$pi.RedirectStandardOutput = $true;

	$p = [System.Diagnostics.Process]::Start($pi);
	$output = $p.StandardOutput.ReadToEnd();
	$p.WaitForExit();

	return $output.split("`n");
}

function UsageExit(){
	$script_name = Split-Path $script:myInvocation.MyCommand.path -Leaf
	write("Usage: " + $script_name + " <command> [parameters]")
	write("  discovery                       : For zabbix low-level discovery")
	write("  get /dev/sd[a-z] ATTRIBUTE_NAME : Get SMART value from device")
	exit 1 # echo %ERRORLEVEL%
}

# Get SMART devices list for zabbix low-level discovery
# Output: JSON
function GetSmartDevicesForZabbixDiscovery(){
	$output = [string[]];
	$line = [string];

	$cmd = GetSMARTCTL
	$output = ExecCmd('"' + $cmd + '" --scan');

	write('{"data":[')
	$f=1
	foreach ($line in $output) {
		$ary = $line.split(" ")
		if ( ($ary[0] -eq '') -or ($ary[2] -eq '') -or ($ary[0] -notmatch '^/dev/sd[\w]+$') ) {
			continue
		}

		if(!$f){
			write(",")
		}
		$f=0
		$devicename = $ary[0].replace("/", "\/")
		$devicetype = $ary[2].replace("/", "\/")
		write('{ "{#DEVICENAME}":"' + $devicename + '","{#DEVICETYPE}":"' + $devicetype + '" }')
	}
	write(']}')
}

# Get SMART values
function GetSmartValue($device, $attribute_name){
	$output = [string[]];
	$line = [string];

	$cmd = GetSMARTCTL
	$output = ExecCmd('"' + $cmd + '" -A ' + $device);
	foreach ($line in $output) {
		$line = $line -replace "^\s+", ""
		$ary = $line -split "\s+"
		if ( $ary[1] -eq $attribute_name ){
			write($ary[9])
			break
		}
	}
}

function main(){
	$command = $script:args[0]
	$param1 = $script:args[1]
	$param2 = $script:args[2]
	
	switch($command){
		'discovery' {
			GetSmartDevicesForZabbixDiscovery
		}
		'get' {
			GetSmartValue $param1 $param2
		}
		default {
			write("[ERROR] Unknown mode: " + $command)
			UsageExit
		}
	}
}

if ( $script:args.length -le 0){
	UsageExit
}

main
